#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from functools import wraps
from flask import make_response, request, session, redirect
from flask import url_for, abort, render_template, jsonify
from werkzeug import secure_filename
from eop import app
from eop import cas
from eop.models.forms import TestForm, DocRamasserForm, DocRendreForm

import xmlrpclib

def get_proxy():
    """Appel du service xmlrpc contenu dans /usr/share/eole/controlevnc/controle_vnc_serveur.py
    """
    return xmlrpclib.ServerProxy(app.config["CONTROLEVNC_URL"])

def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if not session.has_key("username"):
            return redirect(url_for('login', next=request.url))
        else:
            nomfich = os.path.join(app.config["UPLOAD_FOLDER"], ".sessions", session['netid'])
            if not os.path.isfile(nomfich):
                # Cas d'une déconnexion générale de CAS (hors EOP)
                user = cas.CASClient()
                return user.logout()
        return f(*args, **kwargs)
    return decorated_function

class InvalidSecretKeyFile(Exception): pass

def get_username():
    """Vérifie la présence du fichier de session et renvoie le username de la session
       S'il n'existe pas c'est que la session est perdue
    """
    if not session.has_key("netid"):
        return ""
    nomfich = os.path.join(app.config["UPLOAD_FOLDER"], ".sessions", session['netid'])
    if not os.path.isfile(nomfich):
        del(session['netid'])
        del(session['username'])

    if session.has_key("username"):
        return session["username"]
    else:
        return ""

@app.route("/login", methods=["GET", "POST"])
def login():
    """Redirection vers le login CAS
    """
    if request.method == 'POST' and request.form.has_key('logoutRequest'):
        # Requete POST lancée par le SSO lors du logout
        id_logout = cas.parse_tag(request.form['logoutRequest'], "samlp:SessionIndex")
        nomfich = os.path.join(app.config["UPLOAD_FOLDER"], ".sessions", id_logout)
        if os.path.isfile(nomfich): os.remove(nomfich)
        return ""
    else:
        user = cas.CASClient()
        return user.authenticate()

@app.route("/logout")
@login_required
def logout():
    """Déconnexion CAS
    """
    user = cas.CASClient()
    return user.logout()

### GET

@app.route("/documents/rest/groupes", methods = ['GET'])
def get_groups():
    """Récupérer les groupes
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesClasses = proxy.remote_groupes("classe")
    lesNiveaux = proxy.remote_groupes("niveau")
    lesMatieres = proxy.remote_groupes("matiere")
    lesEquipes = proxy.remote_groupes("equipe")
    lesGroupes = proxy.remote_groupes("groupe")
    ret = jsonify({"classes":lesClasses, "matieres":lesMatieres, "niveaux":lesNiveaux, "equipes":lesEquipes, "groupes":lesGroupes})

    return make_response(ret)

@app.route("/documents/rest/membres/<groupe>", methods = ['GET'])
@app.route("/documents/rest/membres/<groupe>/<filtrersur>", methods = ['GET'])
def get_membres(groupe=None, filtrersur=None):
    """Récupérer les membres d'un groupe avec possibilité de filtrer sur un type de membre
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    if filtrersur is None :
        lesMembres =    proxy.remote_membres(groupe)
    else:
        lesMembres =    proxy.remote_membres(groupe, filtrersur)
    return make_response(jsonify({"membres":lesMembres}))

@app.route("/documents/rest/devoirs", methods = ['GET'])
def get_devoirs():
    """Récupérer tous les devoirs d'un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
    return make_response(jsonify(lesDevoirs))

@app.route("/documents/rest/devoirs/aramasser", methods = ['GET'])
def get_devoirs_a_ramasser():
    """Récupérer les devoirs à ramasser d'un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)

    if type(lesDevoirs) == dict:
        for key in lesDevoirs.keys():
            if lesDevoirs[key]["state"] != "dist":
                lesDevoirs.pop(key)
    return make_response(jsonify(lesDevoirs))

@app.route("/documents/rest/devoirs/arendre", methods = ['GET'])
def get_devoirs_a_rendre():
    """Récupérer les devoirs à rendre d'un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)

    if type(lesDevoirs) == dict:
        for key in lesDevoirs.keys():
            if lesDevoirs[key]["state"] != "ram":
                lesDevoirs.pop(key)
    return make_response(jsonify(lesDevoirs))

@app.route("/documents/rest/devoirs/asupprimer", methods = ['GET'])
def get_devoirs_a_supprimer():
    """Récupérer les devoirs à supprimer d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        proxy = get_proxy()
        lesDevoirs = proxy.remote_get_dev_todel(app.config['SECRET_KEY'], username)
        return make_response(jsonify({"DevoirsASupprimer":lesDevoirs}))
    except Exception, e:
        return make_response(jsonify({"erreur":e.message}))

@app.route('/documents/rest/listedoctemp/<typedoc>', methods=['GET'])
def get_liste_doc_temp(typedoc=None):
    """Liste les fichiers en attente pour un prof selon le type de document (devoir ou data)
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        username = secure_filename(username).lower()  # le dossier a été créé dans upload_doc avec cette option donc on le recherhce avec la même option

        if typedoc == "devoir":
            destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
        elif typedoc == "data":
            destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
        else:
            destdir = ""

        if not os.path.isdir(destdir):
            return make_response(jsonify({"listedoc":None}))
        else:
            return make_response(jsonify({"listedoc":sorted(os.listdir(destdir))}))
    except Exception, e:
        return make_response(jsonify({"listedoc":None, "erreur": e}))

@app.route("/documents/rest/checkdevoir/<devname>", methods = ['GET'])
def check_devoir(devname=None):
    """Vérifier l'existence d'un nom de devoir pour un prof
    """
    username = get_username()
    if username == "":
        return make_response(jsonify({"reload":"Session perdue."}))

    proxy = get_proxy()
    lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
    if type(lesDevoirs) == dict:
        retour = devname in lesDevoirs.keys()
    else:
        retour = False
    return make_response(jsonify({"checkdevoir":retour}))

#### POST

@app.route('/documents/uploaddoctemp/<typedoc>', methods=['GET', 'POST'])
def upload_doc_temp(typedoc=None):
    """Upload de fichiers pour un prof selon le type de document (devoir ou data)

        Les fichiers uploadés arrivent un par un ici :
        request.files = ImmutableMultiDict([('file', <FileStorage: u'Antarctique4.jpg' ('image/jpeg')>)]
    """
    if request.method == "POST":
        try:
            username = get_username()
            if username == "":
                return redirect(url_for("login"))

            username = secure_filename(username).lower()

            if typedoc == "devoir":
                destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
            elif typedoc == "data":
                destdir = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
            else:
                raise Exception('upload_doc_temp : typedoc doit être soit "devoir" soit "data"')

            if not os.path.isdir(destdir):
                os.makedirs(destdir)

            for fich in request.files.values():
                ret = "KO"
                if fich:
                    filename = secure_filename(fich.filename).lower()
                    ret = fich.save(os.path.join(destdir, filename))
                    if ret is None: ret = "OK"

            return make_response(jsonify({"uploaddoctemp": ret}))
        except Exception, e:
            return make_response(jsonify({"erreur":e.message}))
    else:
        return abort(404)

@app.route('/documents/supprimerdoctemp/<typedoc>/<fichier>', methods=['GET', 'POST'])
def supprimer_doc_temp(typedoc=None, fichier=None):
    """Suppression d'un fichier en attente pour un prof selon le type de document (devoir ou data)
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        username = secure_filename(username).lower()

        if fichier is None:
            raise Exception("supprimer_doc_temp : Pas de nom de fichier")
        else:
            fichier = secure_filename(fichier).lower()

        if typedoc == "devoir":
            fichier_complet = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username, fichier)
        elif typedoc == "data":
            fichier_complet = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username, fichier)
        else:
            fichier = ""

        ret = "La suppression de %s a échoué." % fichier
        if os.path.isfile(fichier_complet):
            ret = os.remove(fichier_complet)
            if ret is None: ret = "Le fichier %s a été supprimé." % fichier

        return make_response(jsonify({"suppression":ret}))
    except Exception, e:
        return make_response(jsonify({"suppression":e.message}))

@app.route('/documents/viderdoctemp/<typedoc>', methods=['GET', 'POST'])
def vider_doc_temp(typedoc=None):
    """Vidage du dossier temporaire pour un prof selon le type de document (devoir ou data)
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        username = secure_filename(username).lower()

        if typedoc == "devoir":
            dossier = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
        elif typedoc == "data":
            dossier = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)
        else:
            dossier = ""

        if not os.path.isdir(dossier):
            return

        fichiers = os.listdir(dossier)

        for f in fichiers:
            chemin = os.path.join(dossier, f)
            if os.path.isfile(chemin):
                os.remove(chemin)

        ret = ("Le dossier a été vidé." if os.listdir(dossier)==[] else "Le dossier n'a pas pu être vidé.")

        return make_response(jsonify({"vidage":ret}))
    except Exception, e:
        return make_response(jsonify({"vidage":e.message}))

@app.route("/documents/rest/devoirs/distribuer/<devname>/<groupname>/<eleve_only>/<in_perso>", methods = ['GET','POST'])
def devoirs_distribuer(devname=None, groupname=None, eleve_only=None, in_perso=None):
    """Distribuer un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        if devname is None:
            raise Exception("Aucune référence fournie.")
        elif groupname is None:
            raise Exception("Aucun groupe fourni.")
        elif eleve_only is None:
            raise Exception("Aucune informations sur les destinataires fournies (élèves seuls / tous).")
        elif in_perso is None:
            raise Exception("Aucun informations sur la distribution du devoir (immédiat / plus tard).")
        else:
            # Vérification de l'existence de la référence de devoir
            username = secure_filename(username).lower()
            proxy = get_proxy()
            lesDevoirs = proxy.remote_get_devoirs(app.config['SECRET_KEY'], username)
            devname = secure_filename(devname).lower()
            if type(lesDevoirs) == dict:
                if devname in lesDevoirs.keys(): raise Exception("Cette référence existe déjà.")

            # Conversion de "true" ou "false" en True ou False)
            eleve_only = (True if eleve_only == "true" else False)
            in_perso = (True if in_perso == "true" else False)

            # Déplacement des fichiers du devoir vers /home/workgroups/devoirs/<username>/<devname>
            # en envoyant chaque fichiers un par un au service de copie xmlrpc
            dirtempdev = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DEV"], username)
            dirtempdata = os.path.join(app.config['UPLOAD_FOLDER'], app.config["CHEMIN_DAT"], username)

            listefich_dirtempdev = os.listdir(dirtempdev)
            if listefich_dirtempdev == []:
                raise Exception("Aucun document fourni.")    # il doit y avoir au moins un fichier de devoir

            # Traitement du dossier des devoirs
            fichiers = os.listdir(dirtempdev)
            for un_fichier in fichiers:
                chemin = os.path.join(dirtempdev, un_fichier)
                if os.path.isfile(chemin):
                    with open(chemin, "rb") as handle:
                        donnees_fichier = xmlrpclib.Binary(handle.read())
                    proxy.remote_copie_fichier(app.config['SECRET_KEY'], username, devname, "devoir", un_fichier, donnees_fichier)
                    os.remove(chemin)

            # Traitement du dossier des data
            if os.path.isdir(dirtempdata):
                fichiers = os.listdir(dirtempdata)
                for un_fichier in fichiers:
                    chemin = os.path.join(dirtempdata, un_fichier)
                    if os.path.isfile(chemin):
                        with open(chemin, "rb") as handle:
                            donnees_fichier = xmlrpclib.Binary(handle.read())
                        proxy.remote_copie_fichier(app.config['SECRET_KEY'], username, devname, "devoir", un_fichier, donnees_fichier)
                        os.remove(chemin)

            # Appel à l'API de gestion de devoir
            reussi, rateu, dev_name = proxy.remote_distribute(app.config['SECRET_KEY'], username, groupname, devname, eleve_only, in_perso)

            return make_response(jsonify({"distribuer": {"devoir": dev_name, "réussi": reussi, "rate": rateu}}))

    except Exception, e:
        return make_response(jsonify({"erreur":e.message}))

@app.route("/documents/rest/devoirs/ramasser/<devname>", methods = ['GET','POST'])
def devoirs_ramasser(devname=None):
    """Ramasser un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        if devname is None:
            raise Exception("Aucune référence fournie.")
        else:
            proxy = get_proxy()
            reussi, rateu = proxy.remote_ramasse(app.config['SECRET_KEY'], username, devname)

        if rateu:
            raise Exception("Un problème est survenu lors du ramassage.")
        else:
            ramasserDevoir = u"'%s' a bien été ramassé." % devname
    except Exception, e:
        return make_response(jsonify({"erreur":e.message}))
    return make_response(jsonify({"ramasser": ramasserDevoir}))

@app.route("/documents/rest/devoirs/rendre/<devname>", methods = ['GET','POST'])
def devoirs_rendre(devname=None):
    """Rendre un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        if devname is None:
            raise Exception("Aucune référence fournie.")
        else:
            proxy = get_proxy()
            reussi, rateu = proxy.remote_rendre(app.config['SECRET_KEY'], username, devname)

        if rateu:
            raise Exception("Un problème est survenu lors du rendu.")
        else:
            rendreDevoir = u"'%s' a bien été rendu." % devname
    except Exception, e:
        return make_response(jsonify({"erreur":e.message}))
    return make_response(jsonify({"rendre": rendreDevoir}))

@app.route("/documents/rest/devoirs/supprimer/<devname>", methods = ['GET','POST'])
def devoirs_supprimer(devname=None):
    """Supprimer un devoir d'un prof
    """
    try:
        username = get_username()
        if username == "":
            return make_response(jsonify({"reload":"Session perdue."}))

        if devname is None:
            raise Exception("Aucune référence fournie.")
        else:
            proxy = get_proxy()
            supprimerDevoir = proxy.remote_supprimer(app.config['SECRET_KEY'], username, devname)
            if supprimerDevoir:
                ret = u"Les données de '%s' ont été supprimées." % devname
            else:
                ret = u"Les données de '%s' n'ont pas pu être supprimées." % devname
    except Exception, e:
        return make_response(jsonify({"erreur":e.message}))
    return make_response(jsonify({"supprimer": ret}))

### ERREURS

@app.errorhandler(404)
def page_erreur(error):
    return render_template('erreurs/404.html'), error.code

### VUES

@app.route("/documents", methods = ['GET','POST'])
@login_required
def documents():
    """Affichage de la page
    """
    username = get_username()
    proxy = get_proxy()
    est_prof = proxy.remote_est_prof(app.config['SECRET_KEY'], username)

    if not est_prof:
        return render_template('erreurs/non_prof.html', username=username)
    else:
        form = TestForm(request.form)
        form4 = DocRamasserForm(request.form)
        form5 = DocRendreForm(request.form)
        return render_template('documents.html', username=username, form=form, form4=form4, form5=form5)

if __name__ == "__main__":
    app.run(host='192.168.122.202', debug=True )
